@@ -96,6 +96,28 @@ OGRErr OGRXLSXLayer::SyncToDisk()
     return OGRERR_NONE;
 }
 
+/************************************************************************/
+/*                      TranslateFIDFromMemLayer()                      */
+/************************************************************************/
+
+// Translate a FID from MEM convention (0-based) to XLSX convention
+GIntBig OGRXLSXLayer::TranslateFIDFromMemLayer(GIntBig nFID) const
+{
+    return nFID + (1 + (bHasHeaderLine ? 1 : 0));
+}
+
+/************************************************************************/
+/*                        TranslateFIDToMemLayer()                      */
+/************************************************************************/
+
+// Translate a FID from XLSX convention to MEM convention (0-based)
+GIntBig OGRXLSXLayer::TranslateFIDToMemLayer(GIntBig nFID) const
+{
+    if (nFID > 0)
+        return nFID - (1 + (bHasHeaderLine ? 1 : 0));
+    return OGRNullFID;
+}
+
 /************************************************************************/
 /*                          GetNextFeature()                            */
 /************************************************************************/
@@ -103,16 +125,21 @@ OGRErr OGRXLSXLayer::SyncToDisk()
 OGRFeature *OGRXLSXLayer::GetNextFeature()
 {
     Init();
+
     OGRFeature *poFeature = OGRMemLayer::GetNextFeature();
     if (poFeature)
-        poFeature->SetFID(poFeature->GetFID() + 1 +
-                          static_cast<int>(bHasHeaderLine));
+        poFeature->SetFID(TranslateFIDFromMemLayer(poFeature->GetFID()));
     return poFeature;
 }
 
-OGRErr OGRXLSXLayer::CreateField(OGRFieldDefn *poField, int bApproxOK)
+/************************************************************************/
+/*                           CreateField()                              */
+/************************************************************************/
+
+OGRErr OGRXLSXLayer::CreateField(const OGRFieldDefn *poField, int bApproxOK)
 {
     Init();
+
     if (GetLayerDefn()->GetFieldCount() >= 2000)
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -130,29 +157,61 @@ OGRErr OGRXLSXLayer::CreateField(OGRFieldDefn *poField, int bApproxOK)
 OGRFeature *OGRXLSXLayer::GetFeature(GIntBig nFeatureId)
 {
     Init();
-    OGRFeature *poFeature = OGRMemLayer::GetFeature(
-        nFeatureId - (1 + static_cast<int>(bHasHeaderLine)));
+
+    OGRFeature *poFeature =
+        OGRMemLayer::GetFeature(TranslateFIDToMemLayer(nFeatureId));
     if (poFeature)
         poFeature->SetFID(nFeatureId);
     return poFeature;
 }
 
 /************************************************************************/
-/*                           ISetFeature()                               */
+/*                           ISetFeature()                              */
 /************************************************************************/
 
 OGRErr OGRXLSXLayer::ISetFeature(OGRFeature *poFeature)
 {
     Init();
-    if (poFeature == nullptr)
-        return OGRMemLayer::ISetFeature(poFeature);
 
-    GIntBig nFID = poFeature->GetFID();
-    if (nFID != OGRNullFID)
-        poFeature->SetFID(nFID - (1 + static_cast<int>(bHasHeaderLine)));
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin > 0)
+    {
+        const GIntBig nFIDMemLayer = TranslateFIDToMemLayer(nFIDOrigin);
+        if (!GetFeatureRef(nFIDMemLayer))
+            return OGRERR_NON_EXISTING_FEATURE;
+        poFeature->SetFID(nFIDMemLayer);
+    }
+    else
+    {
+        return OGRERR_NON_EXISTING_FEATURE;
+    }
     SetUpdated();
     OGRErr eErr = OGRMemLayer::ISetFeature(poFeature);
-    poFeature->SetFID(nFID);
+    poFeature->SetFID(nFIDOrigin);
+    return eErr;
+}
+
+/************************************************************************/
+/*                         IUpdateFeature()                             */
+/************************************************************************/
+
+OGRErr OGRXLSXLayer::IUpdateFeature(OGRFeature *poFeature,
+                                    int nUpdatedFieldsCount,
+                                    const int *panUpdatedFieldsIdx,
+                                    int nUpdatedGeomFieldsCount,
+                                    const int *panUpdatedGeomFieldsIdx,
+                                    bool bUpdateStyleString)
+{
+    Init();
+
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin != OGRNullFID)
+        poFeature->SetFID(TranslateFIDToMemLayer(nFIDOrigin));
+    SetUpdated();
+    OGRErr eErr = OGRMemLayer::IUpdateFeature(
+        poFeature, nUpdatedFieldsCount, panUpdatedFieldsIdx,
+        nUpdatedGeomFieldsCount, panUpdatedGeomFieldsIdx, bUpdateStyleString);
+    poFeature->SetFID(nFIDOrigin);
     return eErr;
 }
 
@@ -164,16 +223,23 @@ OGRErr OGRXLSXLayer::ICreateFeature(OGRFeature *poFeature)
 {
     Init();
 
-    GIntBig nFID = poFeature->GetFID();
-    if (nFID != OGRNullFID)
+    const GIntBig nFIDOrigin = poFeature->GetFID();
+    if (nFIDOrigin > 0)
     {
-        // Compensate what ISetFeature() will do since
-        // OGRMemLayer::ICreateFeature() will eventually call it
-        poFeature->SetFID(nFID + (1 + static_cast<int>(bHasHeaderLine)));
+        const GIntBig nFIDModified = TranslateFIDToMemLayer(nFIDOrigin);
+        if (GetFeatureRef(nFIDModified))
+        {
+            SetUpdated();
+            poFeature->SetFID(nFIDModified);
+            OGRErr eErr = OGRMemLayer::ISetFeature(poFeature);
+            poFeature->SetFID(nFIDOrigin);
+            return eErr;
+        }
     }
     SetUpdated();
+    poFeature->SetFID(OGRNullFID);
     OGRErr eErr = OGRMemLayer::ICreateFeature(poFeature);
-    poFeature->SetFID(nFID);
+    poFeature->SetFID(TranslateFIDFromMemLayer(poFeature->GetFID()));
     return eErr;
 }
 
@@ -185,8 +251,16 @@ OGRErr OGRXLSXLayer::DeleteFeature(GIntBig nFID)
 {
     Init();
     SetUpdated();
-    return OGRMemLayer::DeleteFeature(nFID -
-                                      (1 + static_cast<int>(bHasHeaderLine)));
+    return OGRMemLayer::DeleteFeature(TranslateFIDToMemLayer(nFID));
+}
+
+/************************************************************************/
+/*                             GetDataset()                             */
+/************************************************************************/
+
+GDALDataset *OGRXLSXLayer::GetDataset()
+{
+    return poDS;
 }
 
 /************************************************************************/
@@ -344,6 +418,18 @@ int OGRXLSXDataSource::Create(const char *pszFilename,
     return TRUE;
 }
 
+/************************************************************************/
+/*                           GetUnprefixed()                            */
+/************************************************************************/
+
+static const char *GetUnprefixed(const char *pszStr)
+{
+    const char *pszColumn = strchr(pszStr, ':');
+    if (pszColumn)
+        return pszColumn + 1;
+    return pszStr;
+}
+
 /************************************************************************/
 /*                           startElementCbk()                          */
 /************************************************************************/
@@ -360,6 +446,8 @@ void OGRXLSXDataSource::startElementCbk(const char *pszNameIn,
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
     switch (stateStack[nStackDepth].eVal)
     {
@@ -400,6 +488,8 @@ void OGRXLSXDataSource::endElementCbk(const char *pszNameIn)
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
 
     nDepth--;
@@ -1057,6 +1147,8 @@ void OGRXLSXDataSource::endElementRow(CPL_UNUSED const char *pszNameIn)
                         {
                             poCurLayer->oSetFieldsOfUnknownType.erase(oIter);
 
+                            auto oTemporaryUnsealer(
+                                poFieldDefn->GetTemporaryUnsealer());
                             poFieldDefn->SetType(eValType);
                             poFieldDefn->SetSubType(eValSubType);
                         }
@@ -1270,6 +1362,8 @@ void OGRXLSXDataSource::startElementSSCbk(const char *pszNameIn,
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
     switch (stateStack[nStackDepth].eVal)
     {
@@ -1305,11 +1399,14 @@ static void XMLCALL endElementSSCbk(void *pUserData, const char *pszNameIn)
     ((OGRXLSXDataSource *)pUserData)->endElementSSCbk(pszNameIn);
 }
 
-void OGRXLSXDataSource::endElementSSCbk(CPL_UNUSED const char *pszNameIn)
+void OGRXLSXDataSource::endElementSSCbk(const char * /*pszNameIn*/)
 {
     if (bStopParsing)
         return;
 
+    // If we were to use pszNameIn, then we need:
+    // pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
 
     nDepth--;
@@ -1453,6 +1550,8 @@ void OGRXLSXDataSource::startElementWBRelsCbk(const char *pszNameIn,
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
     if (strcmp(pszNameIn, "Relationship") == 0)
     {
@@ -1517,18 +1616,6 @@ void OGRXLSXDataSource::AnalyseWorkbookRels(VSILFILE *fpWorkbookRels)
     VSIFCloseL(fpWorkbookRels);
 }
 
-/************************************************************************/
-/*                           GetUnprefixed()                            */
-/************************************************************************/
-
-static const char *GetUnprefixed(const char *pszStr)
-{
-    const char *pszColumn = strchr(pszStr, ':');
-    if (pszColumn)
-        return pszColumn + 1;
-    return pszStr;
-}
-
 /************************************************************************/
 /*                          startElementWBCbk()                         */
 /************************************************************************/
@@ -1545,8 +1632,10 @@ void OGRXLSXDataSource::startElementWBCbk(const char *pszNameIn,
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
-    if (strcmp(GetUnprefixed(pszNameIn), "sheet") == 0)
+    if (strcmp(pszNameIn, "sheet") == 0)
     {
         const char *pszSheetName = GetAttributeValue(ppszAttr, "name", nullptr);
         const char *pszId = GetAttributeValue(ppszAttr, "r:id", nullptr);
@@ -1649,6 +1738,8 @@ void OGRXLSXDataSource::startElementStylesCbk(const char *pszNameIn,
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
     if (strcmp(pszNameIn, "numFmt") == 0)
     {
@@ -1734,6 +1825,8 @@ void OGRXLSXDataSource::endElementStylesCbk(const char *pszNameIn)
     if (bStopParsing)
         return;
 
+    pszNameIn = GetUnprefixed(pszNameIn);
+
     nWithoutEventCounter = 0;
     if (strcmp(pszNameIn, "cellXfs") == 0)
     {
@@ -1799,10 +1892,10 @@ void OGRXLSXDataSource::AnalyseStyles(VSILFILE *fpStyles)
 /*                           ICreateLayer()                             */
 /************************************************************************/
 
-OGRLayer *OGRXLSXDataSource::ICreateLayer(const char *pszLayerName,
-                                          const OGRSpatialReference * /*poSRS*/,
-                                          OGRwkbGeometryType /*eType*/,
-                                          char **papszOptions)
+OGRLayer *
+OGRXLSXDataSource::ICreateLayer(const char *pszLayerName,
+                                const OGRGeomFieldDefn * /*poGeomFieldDefn*/,
+                                CSLConstList papszOptions)
 
 {
     /* -------------------------------------------------------------------- */
@@ -2330,10 +2423,8 @@ static bool WriteLayer(const char *pszName, OGRXLSXLayer *poLayer, int iLayer,
 /*                        WriteSharedStrings()                          */
 /************************************************************************/
 
-static bool
-WriteSharedStrings(const char *pszName,
-                   CPL_UNUSED std::map<std::string, int> &oStringMap,
-                   std::vector<std::string> &oStringList)
+static bool WriteSharedStrings(const char *pszName,
+                               std::vector<std::string> &oStringList)
 {
     CPLString osTmpFilename(
         CPLSPrintf("/vsizip/%s/xl/sharedStrings.xml", pszName));
@@ -2541,7 +2632,7 @@ CPLErr OGRXLSXDataSource::FlushCache(bool /* bAtClosing */)
         bOK &= WriteLayer(pszName, papoLayers[i], i, oStringMap, oStringList);
     }
 
-    bOK &= WriteSharedStrings(pszName, oStringMap, oStringList);
+    bOK &= WriteSharedStrings(pszName, oStringList);
     bOK &= WriteStyles(pszName);
 
     // VSIMkdir(CPLSPrintf("/vsizip/%s/xl/_rels", pszName),0755);
